# This will validate an incoming token and ensure that it is valid, and refresh it if required
#
# It will either return a new access token if it needed to refresh the existing one, None if the token
# was validated and didn't need to be refreshed, or raise an Exception if it can't validate the token
import base64
import datetime
import json
import jwt
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django_cognito import settings
from django_cognito.authentication import utils
from django_cognito.authentication.cognito import constants, actions
from django_cognito.authentication.utils import PublicKey


def validate_token(access_token, refresh_token=None):
    try:
        header, payload = decode_token(access_token)
    except Exception as ex:
        # Invalid token or token we can't decode for whatever reason
        raise Exception("Invalid token")

    public_keys = utils.get_public_keys()

    [matching_key] = [key for key in public_keys['keys'] if key['kid'] == header['kid']]

    if matching_key is None:
        raise Exception("Invalid token public key")
    else:
        # Verify signature using the public key for this pool, as defined the the AWS documentation
        decode = jwt.decode(access_token, PublicKey(matching_key).pem, algorithms=[header['alg']],
                            options={'verify_exp': False})
        pass

    # TODO: Documentation says aud should be the key, but this doesn't exist and client_id has the data aud
    # should have
    #
    # Verify that the audience matches the Cognito app ID, as defined by the AWS documentation
    if payload['client_id'] != constants.CLIENT_ID:
        raise Exception("Invalid token audience")

    # Verify that the issuer matches the URL for the Cognito user pool, as defined by the AWS documentation
    if payload['iss'] != "https://cognito-idp." + constants.POOL_ID.split("_", 1)[0] + ".amazonaws.com/" \
            + constants.POOL_ID:
        raise Exception("Invalid token issuer")

    # Verify that the token is either not expired, or if expired, that we have a refresh token to refresh it
    if payload['exp'] <= datetime.datetime.timestamp(datetime.datetime.utcnow()):
        if not refresh_token:
            # The current access token is expired and no refresh token was provided, authentication fails
            raise Exception("The access token provided has expired. Please login again.")
        else:
            # This token is expired, potentially check for a refresh token? Return this token in the auth return
            # variable?
            result = actions.initiate_auth(payload['username'], constants.REFRESH_TOKEN_FLOW,
                                           refresh_token=refresh_token)

            if result['AuthenticationResult']:
                # Return the freshly generated access token as an indication auth succeeded but a new token was
                # required
                #
                # TODO: DON'T return refresh token here, for methods that require a refresh token we should implement
                # them somewhere else, or differently
                return result['AuthenticationResult']['AccessToken'], refresh_token
            else:
                # Something went wrong with the authentication
                raise Exception("An error occurred while attempting to refresh the access token")
    else:
        # The token validated successfully, we don't need to do anything else here
        return None, None


def decode_token(access_token):
    token_parts = access_token.split(".")

    header = json.loads(
        base64.b64decode(token_parts[0] + "=" * ((4 - len(token_parts[0]) % 4) % 4)).decode('utf-8'))
    payload = json.loads(
        base64.b64decode(token_parts[1] + "=" * ((4 - len(token_parts[1]) % 4) % 4)).decode('utf-8'))

    return header, payload


def process_request(request):
    try:
        access_token = request.COOKIES['AccessToken']
        refresh_token = request.COOKIES['RefreshToken']

        if not access_token or not refresh_token:
            # Need to have this to authenticate, error out
            raise Exception("No valid tokens were found in the request")
        else:
            new_access_token, new_refresh_token = validate_token(access_token, refresh_token)

            header, payload = decode_token(access_token)

            try:
                user = get_user_model().objects.get(username=payload['username'])
            except Exception as ex:
                if settings.AUTO_CREATE_USER:
                    aws_user = actions.admin_get_user(payload['username'])

                    user_attributes = {k: v for dict in [{d['Name']: d['Value']} for d in aws_user['UserAttributes']]
                                       for k, v in dict.items()}

                    user = get_user_model().objects.create(username=payload['username'], email=user_attributes['email'],
                                                           first_name=user_attributes['given_name'],
                                                           last_name=user_attributes['family_name'])

                    user.save()
                else:
                    return AnonymousUser, None, None

        return user, new_access_token, new_refresh_token

    except Exception as ex:
        return AnonymousUser(), None, None



from unittest import TestCase

from django_cognito.authentication.cognito.actions import admin_disable_user, admin_delete_user, admin_confirm_sign_up, \
    admin_update_user_attributes
from django_cognito.authentication.cognito.helpers import initiate_auth, sign_up, forgot_password


class LoginTestCase(TestCase):
    def setUp(self):
        # Ensure this user doesn't already exist before running the rest of the tests
        try:
            result = admin_disable_user("testuser@test.com")
            result = admin_delete_user("testuser@test.com")
        except Exception as ex:
            pass

        self.username = "testuser@test.com"
        self.password = "testpass123"

        self.initialise_test_user()

    def initialise_test_user(self):
        try:
            data = {"username": self.username, "password": self.password,
                    "user_attributes": [{"Name": "given_name", "Value": "Test"},
                                        {"Name": "family_name", "Value": "User"},
                                        {"Name": "email", "Value": "testuser@test.com"}]}

            result = sign_up(data)

            result = admin_confirm_sign_up("testuser@test.com")

            result = admin_update_user_attributes(self.username, [{"Name": "email_verified", "Value": "true"}])
        except Exception as ex:
            self.fail("Could not create a test user for use with other test methods")

    def test_login(self):
        initiate_auth({"username": self.username, "password": self.password, "auth_flow": "USER_PASSWORD_AUTH"})

    def test_password_reset(self):
        forgot_password({"username": self.username})



from django.contrib.auth.models import User, AnonymousUser
from django_cognito.authentication.cognito.helpers import initiate_auth
from django_cognito.authentication.middleware import helpers as m_helpers


# This is utilised when doing username and password validation and calling the authenticate method from
# django.contrib.auth (currently not actually used?)
class AwsDjangoAuthentication:

    def authenticate(self, request, username=None, password=None):
        # This is where we will extract information about the incoming access token from the user,
        # and decide whether or not they are authenticated

        # TODO: Remove this hackiness. Should only return a user from this method as Django seems to only expect a user
        # and not a tuple with a token
        if username is None:
            user, _, _ = m_helpers.process_request(request)

            return user
        else:
            try:
                # We've received a username and password, do that authentication flow instead
                result = initiate_auth({"username":username, "password": password, "auth_flow": "USER_PASSWORD_AUTH"})

                return User.objects.get(email=username)
            except Exception as ex:
                # Either couldn't authenticate or user didn't exist, throw an error
                return AnonymousUser()

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None


from rest_framework.authentication import BaseAuthentication
from django_cognito.authentication.middleware import helpers


# This is utilised by any of the django rest framework views. Currently this is all of the base AWS auth views.
# We probably want to make those views use the default django library to reduce needless dependencies
class AwsRestAuthentication(BaseAuthentication):
    def authenticate(self, request):
        # This is where we will extract information about the incoming access token from the user,
        # and decide whether or not they are authenticated

        user, access_token, refresh_token = helpers.process_request(request)

        # TODO: Potentially create a mixin for views overriding the .finalise_response method to ensure if we
        # end up with a new access token as part of this process, we are able to set it in the response
        #
        # Need some way of setting a new access token or refresh token in the final response
        return user, access_token



import hashlib
import hmac
import base64
import json
import struct
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicNumbers
from urllib.request import urlopen

from django_cognito.authentication.cognito import constants


class PublicKey(object):
    def __init__(self, pubkey):
        self.exponent = self.base64_to_long(pubkey['e'])
        self.modulus = self.base64_to_long(pubkey['n'])
        self.pem = PublicKey.convert(self.exponent, self.modulus)

    def int_array_to_long(self, array):
        return int(''.join(['{:02x}'.format(b) for b in array]), 16)

    def base64_to_long(self, data):
        data = data.encode('ascii')
        _ = base64.urlsafe_b64decode(bytes(data) + b'==')
        return self.int_array_to_long(struct.unpack('%sB' % len(_), _))

    @staticmethod
    def convert(exponent, modulus):
        components = RSAPublicNumbers(exponent, modulus)
        pub = components.public_key(backend=default_backend())
        return pub.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo)


def get_cognito_secret_hash(username: str) -> str:
    message = username + constants.CLIENT_ID
    digest = hmac.new(str(constants.CLIENT_SECRET).encode('UTF-8'), msg=str(message).encode('UTF-8'),
                      digestmod=hashlib.sha256).digest()

    return base64.b64encode(digest).decode()


def get_public_keys():
    public_keys_url = urlopen("https://cognito-idp." + constants.POOL_ID.split("_", 1)[0] + ".amazonaws.com/"
                              + constants.POOL_ID + "/.well-known/jwks.json")
    public_keys = json.loads(public_keys_url.read().decode('utf-8'))

    return public_keys



"""aws_auth URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django_cognito.authentication import views
from django.conf.urls import url

urlpatterns = [
    # Exclude as not appropriate for this app?
    # path('admin/', admin.site.urls),
    url(r'^login', views.initiate_auth),
    # url(r'^logout', views.logout),
    url(r'^signup', views.sign_up),
    url(r'^forgot_password', views.forgot_password),
    url(r'^confirm_signup', views.confirm_sign_up),
    url(r'^confirm_login', views.respond_to_auth_challenge),
    url(r'^confirm_forgot_password', views.confirm_forgot_password)
]



"""
Django settings for aws_auth project.

Generated by 'django-admin startproject' using Django 2.1.1.

For more information on this file, see
https://docs.djangoproject.com/en/2.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.1/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'd-bim@hfv+g^ok=bt6w$nwui&#l@5+gm*e37qyf0uk*xwq^(&d'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_cognito.authentication',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'django_cognito.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


# Database
# https://docs.djangoproject.com/en/2.1/ref/settings/#databases

# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.sqlite3',
#         'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
#     }
# }


# Password validation
# https://docs.djangoproject.com/en/2.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.1/howto/static-files/

STATIC_URL = '/static/'

#########
# PATHS #
#########

# Full filesystem path to the project.
PROJECT_APP_PATH = os.path.dirname(os.path.abspath(__file__))
PROJECT_APP = os.path.basename(PROJECT_APP_PATH)
PROJECT_ROOT = BASE_DIR = os.path.dirname(PROJECT_APP_PATH)

##################
# LOCAL SETTINGS #
##################

# Allow any settings to be defined in local_settings.py which should be
# ignored in your version control system allowing for settings to be
# defined per machine.

# Instead of doing "from .local_settings import *", we use exec so that
# local_settings has full access to everything defined in this module.
# Also force into sys.modules so it's visible to Django's autoreload.

f = os.path.join(PROJECT_APP_PATH, "local_settings.py")
if os.path.exists(f):
    import sys
    import imp

    module_name = "%s.local_settings" % PROJECT_APP
    module = imp.new_module(module_name)
    module.__file__ = f
    sys.modules[module_name] = module
    exec(open(f, "rb").read())



from django_cognito.authentication.cognito import constants, actions

# Collection of methods intended to make the calling of AWS Cognito methods a bit easier. Each method expects both a
# data parameter (which will be a dictionary of values) and an optional param_mapping parameter - another dictionary
# which can be used to override the name of expected values in the data dictionary.
#
# Main use case is when receiving data from a HTTP request - rather than parse it all out individually, just give
# the data to this method with the right naming/mapping, and it'll handle it for you

BAD_DATA_EXCEPTION = "The required parameters were not passed through in the data dictionary"


# TODO: Possibly change some of these methods to just accept one parameter where appropriate (i.e. just a username)


def initiate_auth(data, param_mapping=None):
    if ("username" in data and "password" in data) or ("username" in param_mapping and "password" in param_mapping):
        auth_flow = constants.USER_PASSWORD_FLOW
        username = parse_parameter(data, param_mapping, "username")
        password = parse_parameter(data, param_mapping, "password")

        return actions.initiate_auth(username, auth_flow, password)

    else:
        raise ValueError("Unsupported auth flow")


def respond_to_auth_challenge(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
        challenge_name = parse_parameter(data, param_mapping, 'challenge_name')
        responses = parse_parameter(data, param_mapping, 'responses')
        session = parse_parameter(data, param_mapping, 'session')

    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.respond_to_auth_challenge(username=username, challenge_name=challenge_name,
                                             responses=responses, session=session)


def sign_up(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
        password = parse_parameter(data, param_mapping, 'password')
        user_attributes= parse_parameter(data, param_mapping, 'user_attributes')

    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.sign_up(username, password, user_attributes)


def confirm_sign_up(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
        confirmation_code = parse_parameter(data, param_mapping, 'password')
        force_alias_creation = parse_parameter(data, param_mapping, 'force_alias_creation')

    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.confirm_sign_up(username, confirmation_code, force_alias_creation)


def forgot_password(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')

    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.forgot_password(username)


def confirm_forgot_password(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
        new_password = parse_parameter(data, param_mapping, 'new_password')
        code = parse_parameter(data, param_mapping, 'code')

    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.confirm_forgot_password(username, code, new_password)


def admin_get_user(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')

    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.admin_get_user(username)


def admin_update_user_attributes(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
        user_attributes = parse_parameter(data, param_mapping, 'user_attributes')
    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.admin_update_user_attributes(username, user_attributes)


def admin_disable_user(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.admin_disable_user(username)


def admin_delete_user(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.admin_delete_user(username)

def admin_create_user(data, param_mapping=None):
    try:
        username = parse_parameter(data, param_mapping, 'username')
        user_attributes = parse_parameter(data, param_mapping, 'user_attributes')
        temporary_password = parse_parameter(data, param_mapping, 'temporary_password')

        if "suppress" in data or "suppress" in param_mapping:
            supress = parse_parameter(data, param_mapping, 'suppress')

    except Exception as ex:
        raise ValueError(BAD_DATA_EXCEPTION)

    return actions.admin_create_user(username, user_attributes, temporary_password)




def parse_parameter(data, param_mapping, param=None):
    if param_mapping is not None:
        if param in param_mapping:
            return data[param_mapping[param]]
    else:
        return data[param]



from django.conf import settings
import boto3


class CognitoClient:
    client = boto3.client('cognito-idp', aws_access_key_id=settings.AWS_ACCESS_KEY,
                          aws_secret_access_key=settings.AWS_SECRET_KEY)


class CognitoException(Exception):
    def __init__(self, message, status):
        super(CognitoException, self).__init__(message)

        self.status = status

    @staticmethod
    def create_from_exception(ex):
        return CognitoException({'error': ex.response['Error']['Message']},
                                ex.response['ResponseMetadata']['HTTPStatusCode'])



from django.conf import settings
from django_cognito.authentication.cognito.base import CognitoClient

USER_PASSWORD_FLOW = 'USER_PASSWORD_AUTH'
REFRESH_TOKEN_FLOW = 'REFRESH_TOKEN'
REFRESH_TOKEN_AUTH_FLOW = 'REFRESH_TOKEN_AUTH'

SMS_CHALLENGE = 'SMS_MFA'
SOFTWARE_CHALLENGE = 'SOFTWARE_TOKEN_MFA'
NEW_PASSWORD_CHALLENGE = 'NEW_PASSWORD_REQUIRED'

CLIENT_ID = settings.APP_CLIENT_ID
CLIENT_SECRET = settings.APP_SECRET_KEY
POOL_ID = settings.COGNITO_POOL_ID
AWS_ACCESS_KEY = settings.AWS_ACCESS_KEY
AWS_SECRET_KEY = settings.AWS_SECRET_KEY

AWS_EXCEPTIONS = (
    CognitoClient.client.exceptions.AliasExistsException,
    CognitoClient.client.exceptions.CodeDeliveryFailureException,
    CognitoClient.client.exceptions.CodeMismatchException,
    CognitoClient.client.exceptions.ExpiredCodeException,
    CognitoClient.client.exceptions.InternalErrorException,
    CognitoClient.client.exceptions.InvalidEmailRoleAccessPolicyException,
    CognitoClient.client.exceptions.InvalidLambdaResponseException,
    CognitoClient.client.exceptions.InvalidParameterException,
    CognitoClient.client.exceptions.InvalidPasswordException,
    CognitoClient.client.exceptions.InvalidSmsRoleAccessPolicyException,
    CognitoClient.client.exceptions.InvalidSmsRoleTrustRelationshipException,
    CognitoClient.client.exceptions.InvalidUserPoolConfigurationException,
    CognitoClient.client.exceptions.LimitExceededException,
    CognitoClient.client.exceptions.MFAMethodNotFoundException,
    CognitoClient.client.exceptions.NotAuthorizedException,
    CognitoClient.client.exceptions.PasswordResetRequiredException,
    CognitoClient.client.exceptions.ResourceNotFoundException,
    CognitoClient.client.exceptions.SoftwareTokenMFANotFoundException,
    CognitoClient.client.exceptions.TooManyRequestsException,
    CognitoClient.client.exceptions.UnexpectedLambdaException,
    CognitoClient.client.exceptions.UserLambdaValidationException,
    CognitoClient.client.exceptions.UserNotConfirmedException,
    CognitoClient.client.exceptions.UserNotFoundException,
    CognitoClient.client.exceptions.UsernameExistsException
)



import json
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods
from django_cognito.authentication.cognito import helpers
from django_cognito.authentication.cognito.base import CognitoException


@csrf_exempt
@require_http_methods(['POST'])
def initiate_auth(request):
    try:
        data = json.loads(request.body.decode('utf-8'))
        result = helpers.initiate_auth(data)

        return JsonResponse(result)
    except CognitoException as ex:
        return JsonResponse(ex.args[0], status=ex.status)
    except ValueError as ex:
        return JsonResponse({"error": ex.args[0]}, status=400)


@require_http_methods(['POST'])
def respond_to_auth_challenge(request):
    try:
        data = json.loads(request.body.decode('utf-8'))
        result = helpers.respond_to_auth_challenge(data)

        return JsonResponse(result)
    except CognitoException as ex:
        return JsonResponse(ex.args[0], status=ex.status)
    except ValueError as ex:
        return JsonResponse({"error": ex.args[0]}, status=400)
    pass


@csrf_exempt
@require_http_methods(['POST'])
def forgot_password(request):
    try:
        data = json.loads(request.body.decode('utf-8'))
        result = helpers.forgot_password(data)

        return JsonResponse(result)
    except CognitoException as ex:
        return JsonResponse(ex.args[0], status=ex.status)
    except ValueError as ex:
        return JsonResponse({"error": ex.args[0]}, status=400)
    pass


@csrf_exempt
@require_http_methods(['POST'])
def confirm_forgot_password(request):
    try:
        data = json.loads(request.body.decode('utf-8'))
        result = helpers.confirm_forgot_password(data)

        return JsonResponse(result)
    except CognitoException as ex:
        return JsonResponse(ex.args[0], status=ex.status)
    except ValueError as ex:
        return JsonResponse({"error": ex.args[0]}, status=400)
    pass


@csrf_exempt
@require_http_methods(['POST'])
def sign_up(request):
    try:
        data = json.loads(request.body.decode('utf-8'))
        result = helpers.sign_up(data)

        return JsonResponse(result)
    except CognitoException as ex:
        return JsonResponse(ex.args[0], status=ex.status)
    except ValueError as ex:
        return JsonResponse({"error": ex.args[0]}, status=400)
    pass


@csrf_exempt
@require_http_methods(['POST'])
def confirm_sign_up(request):
    try:
        data = json.loads(request.body.decode('utf-8'))
        result = helpers.confirm_sign_up(data)

        return JsonResponse(result)
    except CognitoException as ex:
        return JsonResponse(ex.args[0], status=ex.status)
    except ValueError as ex:
        return JsonResponse({"error": ex.args[0]}, status=400)
    pass



from django.conf import settings
from django_cognito.authentication.middleware import helpers


# This is utilised from normal Django views. Currently used for anything that requires authentication but isn't
# already utilising rest framework
class AwsDjangoMiddleware:
    def __init__(self, get_response=None):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Get the user and a new token if required
        user, token, refresh_token = helpers.process_request(request)

        request.user = user

        response = self.get_response(request)

        if token:
            # TODO: Set the token in the response here as well? If the user hits here, they're still active
            http_only = settings.HTTP_ONLY_COOKIE
            secure = settings.SECURE_COOKIE

            response.set_cookie(key='AccessToken', value=token,
                                secure=secure, httponly=http_only)
            response.set_cookie(key="RefreshToken", value=refresh_token,
                                secure=secure, httponly=http_only)
            pass

        return response

    def process_request(self, request):
        pass



from setuptools import setup, find_packages
from django_cognito import __version__ as version

setup(
    name='django_cognito',
    version=version,
    packages=find_packages(),
    url='https://github.com/Olorin92/django_cognito',
    license='',
    author='Alex Plant',
    author_email='alex.c.plant@gmail.com',
    description='Library for allowing the use of AWS Cognito security in Django projects',
    install_requires=[
        "asn1crypto==0.23.0",
        "boto3==1.8.2",
        "botocore==1.11.2",
        "cryptography>=2.3",
        "Django>=1.10.0",
        "djangorestframework==3.6.4",
        "idna==2.6",
        "PyJWT==1.6.4",
        "python-dateutil==2.6.1",
        "urllib3==1.22",
    ]
)



import json
from urllib.request import urlopen

from django_cognito.authentication import utils
from django_cognito.authentication.cognito import constants
from django_cognito.authentication.cognito.base import CognitoClient, CognitoException


# Should return an AccessToken and a RefreshToken (with those keys) if successful,
# otherwise return challenge information if a challenge is required
#
# AccessToken and RefreshToken will both be keys in the AuthenticationResult key
def initiate_auth(username, auth_flow, password=None, refresh_token=None, srp_a=None):
    auth_parameters = {'SECRET_HASH': utils.get_cognito_secret_hash(username)}

    if auth_flow == constants.USER_PASSWORD_FLOW:
        auth_parameters['USERNAME'] = username
        auth_parameters['PASSWORD'] = password
    elif auth_flow == constants.REFRESH_TOKEN_AUTH_FLOW or auth_flow == constants.REFRESH_TOKEN_FLOW:
        if refresh_token is None:
            raise Exception("To use the refresh token flow you must provide a refresh token")
        else:
            auth_parameters['REFRESH_TOKEN'] = refresh_token
    else:
        raise Exception("Provided auth flow is not supported")

    try:
        return CognitoClient.client.initiate_auth(AuthFlow=auth_flow, ClientId=constants.CLIENT_ID,
                                                  AuthParameters=auth_parameters)

    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def respond_to_auth_challenge(username, challenge_name, responses, session=None):
    responses['USERNAME'] = username
    responses['SECRET_HASH'] = utils.get_cognito_secret_hash(username)

    try:
        return CognitoClient.client.respond_to_auth_challenge(ClientId=constants.CLIENT_ID,
                                                              ChallengeName=challenge_name,
                                                              ChallengeResponses=responses,
                                                              Session=session)

    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def sign_up(username, password, user_attributes, validation_data=None):
    secret_hash = utils.get_cognito_secret_hash(username)
    kwargs = {"ClientId": constants.CLIENT_ID, "SecretHash": secret_hash,
              "Username": username, "Password": password,
              "UserAttributes": user_attributes}

    if validation_data:
        kwargs['ValidationData'] = validation_data

    try:
        return CognitoClient.client.sign_up(**kwargs)

    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def confirm_sign_up(username, confirmation_code, force_alias_creation=False):
    secret_hash = utils.get_cognito_secret_hash(username)

    try:
        return CognitoClient.client.confirm_sign_up(ClientId=constants.CLIENT_ID, SecretHash=secret_hash,
                                                    Username=username, ForceAliasCreation=force_alias_creation,
                                                    ConfirmationCode=confirmation_code)
    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def forgot_password(username):
    secret_hash = utils.get_cognito_secret_hash(username)

    try:
        return CognitoClient.client.forgot_password(ClientId=constants.CLIENT_ID, SecretHash=secret_hash,
                                                    Username=username)
    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def confirm_forgot_password(username, code, new_password):
    secret_hash = utils.get_cognito_secret_hash(username)

    try:
        return CognitoClient.client.confirm_forgot_password(ClientId=constants.CLIENT_ID, SecretHash=secret_hash,
                                                            Username=username,
                                                            Password=new_password,
                                                            ConfirmationCode=code)
    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def admin_get_user(username):
    try:
        return CognitoClient.client.admin_get_user(UserPoolId=constants.POOL_ID, Username=username)
    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def admin_disable_user(username):
    result = CognitoClient.client.admin_disable_user(UserPoolId=constants.POOL_ID, Username=username)

    return result


def admin_delete_user(username):
    result = CognitoClient.client.admin_delete_user(UserPoolId=constants.POOL_ID, Username=username)

    return result


def admin_confirm_sign_up(username):
    result = CognitoClient.client.admin_confirm_sign_up(UserPoolId=constants.POOL_ID, Username=username)

    return result


def admin_create_user(username, user_attributes, temporary_password, suppress=False):
    message_action = 'SUPPRESS' if suppress else 'RESEND'

    result = CognitoClient.client.admin_create_user(UserPoolId=constants.POOL_ID, Username=username,
                                                    TemporaryPassword=temporary_password, MessageAction=message_action,
                                                    UserAttributes=user_attributes)

    return result


def admin_update_user_attributes(username, user_attributes):
    try:
        return CognitoClient.client.admin_update_user_attributes(UserPoolId=constants.POOL_ID, Username=username,
                                                                 UserAttributes=user_attributes)
    except constants.AWS_EXCEPTIONS as ex:
        raise CognitoException.create_from_exception(ex)


def resend_confirmation_code(username):
    result = CognitoClient.client.resend_confirmation_code(ClientId=constants.CLIENT_ID, Username=username,
                                                           SecretHash=utils.get_cognito_secret_hash(username))

    return result


def admin_list_users(attributes_to_get=None, pagination_token=None):
    args = {'UserPoolId': constants.POOL_ID}

    if attributes_to_get:
        args['AttributesToGet'] = attributes_to_get
    if pagination_token:
        args['PaginationToken'] = pagination_token

    result = CognitoClient.client.list_users(**args)

    return result
